package cz.cvut.kbss.reporting.service.repository;

import cz.cvut.kbss.reporting.model.OccurrenceReport;
import cz.cvut.kbss.reporting.model.Person;
import cz.cvut.kbss.reporting.model.Tag;
import cz.cvut.kbss.reporting.persistence.dao.GenericDao;
import cz.cvut.kbss.reporting.persistence.dao.OccurrenceReportDao;
import cz.cvut.kbss.reporting.service.security.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

@Service
public class OccurrenceReportService extends AbstractRepositoryService<OccurrenceReport> {

    private static final String DEFAULT_USERNAME = "default@ear.cz";

    private final OccurrenceReportDao dao;

    private final PersonService personService;

    private final SecurityUtils securityUtils;

    private Person defaultUser = generateUser();

    private static Person generateUser() {
        final Person p = new Person();
        p.setUsername("default@ear.cz");
        p.setFirstName("Default");
        p.setLastName("Person");
        p.setPassword("12345");
        return p;
    }

    @Autowired
    public OccurrenceReportService(OccurrenceReportDao dao, PersonService personService, SecurityUtils securityUtils) {
        this.dao = dao;
        this.personService = personService;
        this.securityUtils = securityUtils;
    }

    @PostConstruct
    private void defaultUser() {
        if (personService.exists(DEFAULT_USERNAME)) {
            this.defaultUser = personService.findByUsername(DEFAULT_USERNAME);
        } else {
            this.defaultUser = generateUser();
            personService.persist(defaultUser);
        }
    }

    public List<OccurrenceReport> findAll(Tag tag) {
        return dao.findAll(tag);
    }


    @Override
    protected GenericDao<OccurrenceReport> getPrimaryDao() {
        return dao;
    }

    @Override
    void prePersist(OccurrenceReport instance) {
        persistTags(instance);
        instance.setAuthor(securityUtils.getCurrentUser());
        instance.setCreated(LocalDateTime.now());
    }

    @Override
    void preUpdate(OccurrenceReport instance) {
        persistTags(instance);
        // A bug in Eclipselink causes the defaultUser to be persisted even though no cascading is configured on the lastEditor attribute
        instance.setLastEditor(defaultUser);
        instance.setLastModified(LocalDateTime.now());
    }

    private void persistTags(OccurrenceReport instance) {
        Optional.ofNullable(instance.getTags())
                .ifPresent(
                        tags -> tags.stream()
                                    .filter(t -> t.getAuthor() == null)
                                    .forEach(
                                            t -> {
                                                t.setCreated(LocalDateTime.now());
                                                t.setAuthor(securityUtils.getCurrentUser());
                                            }
                                    )
                );
    }
}
