import React from "react";
import PropTypes from "prop-types";
import {FormattedMessage} from "react-intl";
import I18nWrapper from "../../i18n/I18nWrapper";
import injectIntl from "../../utils/injectIntl";
import Utils from "../../utils/Utils";

const ReportProvenance = (props) => {
    const report = props.report;
    if (report.isNew) {
        return null;
    }
    const author = report.author ? report.author.firstName + ' ' + report.author.lastName : '',
        created = Utils.formatDate(new Date(report.created));
    let lastEditor, lastModified;
    if (!report.lastModified) {
        return <div className='notice-small'>
            <FormattedMessage id='report.created-by-msg'
                              values={{date: created, name: <b>{author}</b>}}/>
        </div>;
    }
    lastEditor = report.lastEditor ? report.lastEditor.firstName + ' ' + report.lastEditor.lastName : '';
    lastModified = Utils.formatDate(new Date(report.lastModified));
    return <div className='notice-small'>
        <FormattedMessage id='report.created-by-msg'
                          values={{date: created, name: <b>{author}</b>}}/>
        &nbsp;
        <FormattedMessage id='report.last-edited-msg'
                          values={{date: lastModified, name: <b>{lastEditor}</b>}}/>
    </div>;
};

ReportProvenance.propTypes = {
    report: PropTypes.object.isRequired
};

export default injectIntl(I18nWrapper(ReportProvenance));
